precision mediump float;

uniform float uPhase;
uniform vec3 uLightDirection;
uniform vec3 uRingPlaneNormal; // Represents the "up" or plane normal of the plane the planets rings lie on
uniform vec3 uAmbientColor;
uniform vec3 uExposure;

//Gamma parameters
const float fExposure = 3.0;
const vec3 v3Exposure = vec3(-fExposure, -fExposure, -fExposure);

varying vec2 TexCoord;
varying vec3 normal;
varying vec3 viewDir;
varying vec3 wPos;

uniform sampler2D diffuseTexture;
uniform sampler2D atmos_frontcolor;
uniform sampler2D atmos_attenuate;
uniform sampler2D ringTexture; 
//Ring texture channels should contain:
	//R = Front scatter
	//G = Back scatter
	//B = Unlit
	//A = Alpha 


const vec3 v3_ONE = vec3(1.0,1.0,1.0);

void getLookupLighting(sampler2D atmos_frontcolor_sampler, 
						sampler2D atmos_attenuate_sampler, 
						vec3 normal, 
						vec3 viewDir,
						out vec3 c0,
						out vec3 c1)
{
	float tc_light = dot(normal,uLightDirection);
	tc_light = (1.0 - (tc_light * 0.5 + 0.5));
	
	vec3 normalised_viewDir = normalize(viewDir);
	float tc_view = dot(normal,normalised_viewDir);
	tc_view = 1.0-clamp(tc_view,0.0,1.0);

	vec2 atmosTexLookup = vec2(tc_view,tc_light);
	c0 = texture2D(atmos_frontcolor_sampler,atmosTexLookup).rgb;
	c0 *= c0;
	c1 = texture2D(atmos_attenuate_sampler,atmosTexLookup).rgb;
	c1 *= c1;
}



//Performs a simple disc collision check and returns a value between 0 and 1, suitable
//for use as a texture coordinate.
float getDiscIntersection(vec3 v3Pos,vec3 v3Ray)
{
	//CONSTANTS
	//Ring radius values need to be in unit space. IE, relative to a sphere with a radius of 1.
	const float innerRadius = 1.1611;
	const float innerRadius2 = innerRadius*innerRadius;
	const float outerRadius = 2.3258;
	const float outerRadius2 = outerRadius*outerRadius;
	const float inverseRingWidth2 = 1.0 / (outerRadius2 - innerRadius2);
	//const vec3 planeNormal = vec3(0.0,1.0,0.0);
	
	float planeDistance = dot(vec3(0.0),uRingPlaneNormal);// Usually Dot a_PlanePos a_PlaneNormal but can be simplified here
	float pointDistance = dot(v3Pos, uRingPlaneNormal);
	
	float t = (planeDistance - pointDistance) / dot(v3Ray, uRingPlaneNormal);
	t = max(0.0,t);// Negative t values need to be handled
	vec3 collisionPoint = v3Pos + (t * v3Ray);
	
	float distSquared = dot(collisionPoint, collisionPoint);
	
	float tc = (distSquared - innerRadius2) * inverseRingWidth2;
	return clamp(tc,0.0,1.0);
}


void main()
{

	//Ring shadow
	float discShadow = getDiscIntersection(wPos,uLightDirection);
	vec2 shadowUV = vec2(discShadow,0.0);
	float shadowFac = 1.0-texture2D(ringTexture,shadowUV).a;
	
	//Lookup lighting
	vec3 c0;
	vec3 c1;
	
	getLookupLighting(atmos_frontcolor,atmos_attenuate,normal,viewDir,c0,c1);
	
	
	vec3 v3DiffuseColor = texture2D(diffuseTexture,TexCoord).rgb;
	v3DiffuseColor = v3DiffuseColor * v3DiffuseColor; // Approximate gamma correction
	
	vec4 final;
	final.rgb = (c0 + v3DiffuseColor * c1) * vec3(shadowFac);
	final.rgb += v3DiffuseColor * uAmbientColor;
	
	//exposure (expensive)
	//final.rgb = v3_ONE - exp(v3Exposure * final.rgb);
	
	//S curve exposure, taken from http://kajon.se/portfolio/Ibr/RealtimeTonemapper_rapport.pdf section 3.2
	//final.rgb = final.rgb / (final.rgb + uExposure);
	//final.rgb = c0;
	//gamma correction back to 2.0 space
	//final.rgb = sqrt(final.rgb);
	
	//filmic tone mapping
	final.rgb *= uExposure;
	final.rgb = max(vec3(0.0),final.rgb - vec3(0.004));
	final.rgb = (final.rgb*(vec3(6.2)*final.rgb+vec3(0.5))) / (final.rgb*(vec3(6.2)*final.rgb+vec3(1.7))+vec3(0.06));
	
	final.a = 1.0;
	
	gl_FragColor = final;
}